angular.module('beamng.apps')
.directive('velocityTacho', ["StreamsManager", "UiUnits", "bngApi", function(StreamsManager, UiUnits, bngApi) {
  // noinspection CssUnknownTarget
  return {
    template:
        '<div id="VT_container">' +
          '<link rel="stylesheet" href="../ui/modules/apps/VelocityTacho/app.css">' +
          '<link rel="stylesheet" href="../ui/tachometers/base.css">' +
          '<canvas class="canvas" id="VT_bgCanvas" width="300" height="300"></canvas>' +
          '<canvas class="canvas" id="VT_canvas"   width="300" height="300"></canvas>' +
          // '<div ng-include="\'../ui/tachometers/colours.html\'"></div>' +
        '</div>',
    replace: true,
    restrict: 'EA',
    link: function ($scope, $element) {
      // The internal ID for this app
      const ID = "VT";
      const REQ_STREAMS = ["electrics", "engineInfo"];
      const REQ_SERVICES = ["service", "velocityService"];

      StreamsManager.add(REQ_STREAMS);


      let scriptsLoaded = 0;
      // Add in the script files for the base service
      for (let service of REQ_SERVICES) {
        let script = document.createElement("script");
        script.src = "../ui/tachometers/" + service + ".js";
        // Add in this directives ID to avoid conflicts between all of these modded apps
        script.id = service + ID;
        $element.append(script);

        document.getElementById(script.id).onload = function() {
          scriptsLoaded++;
          // Wait for all scripts to load
          if (scriptsLoaded === REQ_SERVICES.length)
            onload();
        };
      }


      let app = null;
      let TachoBase = null;

      //////////////////////////////
      // Rendering/Main Functions //
      //////////////////////////////

      function getMaxEngineRPM(engineInfo) {
        return engineInfo[1];
      }

      /**
       * Returns a list of all dynamic variables that will/may change between frames for the vehicle
       * @param {Object} engineInfo - the engine info object from the engineInfo stream
       * @param {Object} electrics - the electrics object from the electrics stream
       * @returns {{currSpeed: number, currRPM: number, lights: number, currGear: number,
       * pBrakeEngaged: boolean, waterTemp: number, fuel: number}} the list of variables
       */
      function getDynamicVars(engineInfo, electrics) {
        let lights = 0;
        if (electrics.highbeam)
          lights = 2;
        else if (electrics.lowbeam)
          lights = 1;

        return {
          currGear: engineInfo[16],
          currSpeed: electrics.wheelspeed,
          currRPM: engineInfo[4],
          lights: lights,
          pBrakeEngaged: electrics.parkingbrake > 0,
          waterTemp: electrics.watertemp,
          fuel: engineInfo[11] / engineInfo[12]
        };
      }


      $scope.$on("streamsUpdate", function (event, streams) {
        let engineInfo = streams.engineInfo;
        let electrics = streams.electrics;

        // Ensure engineInfo exists AND ALSO that the engine is actually setup at this point
        let isEngineSetup = engineInfo && getMaxEngineRPM(engineInfo) !== 0;

        // Wait until everything is ready
        if (!app || app.state === app.states.preInit || !isEngineSetup || !electrics)
          return;

        let baseVars = getDynamicVars(engineInfo, electrics);

        // Perform first time setup if necessary
        if (app.state === app.states.init) {
          app.initFields(
              baseVars.currGear,
              baseVars.currSpeed,
              baseVars.currRPM,
              getMaxEngineRPM(engineInfo),
              baseVars.lights,
              baseVars.waterTemp,
              baseVars.fuel
          );
          app.load(bngApi);
          app.drawBackgroundCanvas();
          app.state = app.states.loaded;
        }

        app.updateVariables(
            baseVars.currGear,
            baseVars.currSpeed,
            baseVars.currRPM,
            baseVars.lights,
            baseVars.pBrakeEngaged,
            baseVars.waterTemp,
            baseVars.fuel
        );
        app.draw();
      });

      $scope.$on("$destroy", () => StreamsManager.remove(REQ_STREAMS));
      $scope.$on("VehicleFocusChanged", () => TachoBase.TachometerApp.reload(app, true));
      $scope.$on("VehicleChange", () => TachoBase.TachometerApp.reload(app, false));

      $scope.$on('app:resized', function (event, data) {
        // TachoBase.TachometerApp.changeSize(app, data.width, data.height);
      });

      /**
       * Loads the external services and initialises some base constants.
       */
      function onload() {
        // Request and setup the services
        let injector = angular.injector(["beamng.apps"]);
        let ModBase = injector.get("ModVelocityBase");
        TachoBase = ModBase.TachoBase;

        app = new ModBase.VelocityTachoApp(ID, document, bngApi, UiUnits);
      }
    }
  };
}]);